module Njord
  module Commands
    class KubernetesSetup < Thor
      include Thor::Actions
      attr_accessor :config

      APPNAME = "njord-blog"
      NAMESPACE = "njord-namespace"
      ENVIRONMENT = "production"
      HOSTNAME = "blog.njord.com"

      desc "all", "Configures all resources needed to run a rails project"
      def all
        init
        setup(:deployment)
        setup(:service)
        setup(:configmap)
        setup(:ingress)
      end

      desc "deployment", "Configures the deployment resource"
      def deployment
        init
        setup(:deployment)
      end

      desc "service", "Configures the service resource"
      def service
        init
        setup(:service)
      end

      desc "configmap", "Configures the configmap resource"
      def configmap
        init
        setup(:configmap)
      end

      desc "ingress", "Configures the ingress resource"
      def ingress
        init
        setup(:ingress)
      end

      def self.banner(command, _namespace = nil, _subcommand = false)
        "#{basename} k8s setup #{command.usage}"
      end

      def self.source_root
        "#{__dir__}/../templates/kubernetes"
      end

      private
      def init
        @config = {}
        @config[:appname] = ask("Name of your app [#{APPNAME}]", default: APPNAME)
        @config[:namespace] = ask("Namespace of your app [#{NAMESPACE}]", default: NAMESPACE)
        @config[:environment] = ask("Environment your app is running in [#{ENVIRONMENT}]", default: ENVIRONMENT)
        @config[:hostname] = ask("Hostname of your app [#{HOSTNAME}]", default: HOSTNAME)
        @config[:imagename] = ask("Name of your Dockerimage including repo [#{full_image_name}]", default: full_image_name)
      end

      def setup(resource)
        case resource
        when :deployment then template("deployments.yaml.tt", "deployment/deployments.yaml", @config)
        when :service then template("services.yaml.tt", "deployment/services.yaml", @config)
        when :configmap then template("configmaps.yaml.tt", "deployment/configmaps.yaml", @config)
        when :ingress then template("ingresses.yaml.tt", "deployment/ingresses.yaml", @config)
        end
      end

      def full_image_name
        image = Njord.config.images.first
        "#{image.docker_repo}/#{image.image_name}:latest"
      end
    end
  end
end
