require 'open3'

module Njord
  class DockerHandler
    class << self
      def build_images
        Njord.config.images.each do |image_config|
          execute("docker build #{build_args(image_config)}-t #{image_config.image_name}:#{image_tag(image_config)}" \
            " -f #{image_config.dockerfile} .")
        end
      end

      def tag_images
        Njord.config.images.each do |image_config|
          execute("docker tag #{image_config.image_name}:#{image_tag(image_config)} #{image_config.docker_repo}/#{image_config.image_name}:#{image_tag(image_config)}")
          execute("docker tag #{image_config.image_name}:#{image_tag(image_config)} #{image_config.docker_repo}/#{image_config.image_name}:latest")
        end
      end

      def push_images
        Njord.config.images.each do |image_config|
          execute("docker push #{image_config.docker_repo}/#{image_config.image_name}:#{image_tag(image_config)}")
          execute("docker push #{image_config.docker_repo}/#{image_config.image_name}:latest")
        end
      end

      private
      def execute(cmd)
        puts "Executing #{cmd}"
        raise RuntimeError unless system(cmd, out: $stdout, err: :out)
      end

      def image_tag(image_config)
        case image_config.tagging
        when :sem_ver then "" # TODO
        else
          image_tag_via_git
        end
      end

      def image_tag_via_git
        `git rev-parse --short HEAD`.strip
      end

      def build_args(image_config)
        image_config.build_args && image_config.build_args.collect{ |x| "--build-arg #{x} " }.join("")
      end
    end
  end
end
